@@ -17,7 +17,7 @@
 	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-/* $Id: ifdhandler.c 6563 2013-03-06 14:27:02Z rousseau $ */
+/* $Id: ifdhandler.c 6753 2013-09-21 14:16:17Z rousseau $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -126,6 +126,7 @@ static RESPONSECODE CreateChannelByNameOrChannel(DWORD Lun,
 	{
 		unsigned char pcbuffer[SIZE_GET_SLOT_STATUS];
 		unsigned int oldReadTimeout;
+		RESPONSECODE cmd_ret;
 		_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
 		/* Maybe we have a special treatment for this reader */
@@ -138,9 +139,12 @@ static RESPONSECODE CreateChannelByNameOrChannel(DWORD Lun,
 		 * few tries. It is an empirical hack */
 
 		/* The reader may have to start here so give it some time */
-		ret = CmdGetSlotStatus(reader_index, pcbuffer);
-		if (IFD_NO_SUCH_DEVICE == ret)
-			return ret;
+		cmd_ret = CmdGetSlotStatus(reader_index, pcbuffer);
+		if (IFD_NO_SUCH_DEVICE == cmd_ret)
+		{
+			return_value = cmd_ret;
+			goto error;
+		}
 
 		/* save the current read timeout computed from card capabilities */
 		oldReadTimeout = ccid_descriptor->readTimeout;
@@ -156,9 +160,6 @@ static RESPONSECODE CreateChannelByNameOrChannel(DWORD Lun,
 		}
 		else
 		{
-			/* set back the old timeout */
-			ccid_descriptor->readTimeout = oldReadTimeout;
-
 			/* Maybe we have a special treatment for this reader */
 			return_value = ccid_open_hack_post(reader_index);
 			if (return_value != IFD_SUCCESS)
@@ -166,6 +167,9 @@ static RESPONSECODE CreateChannelByNameOrChannel(DWORD Lun,
 				DEBUG_CRITICAL("failed");
 			}
 		}
+
+		/* set back the old timeout */
+		ccid_descriptor->readTimeout = oldReadTimeout;
 	}
 
 error:
@@ -505,10 +509,6 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 
 				ccid_desc = get_ccid_descriptor(reader_index);
 
-				/* more than one slot is not supported */
-				if (ccid_desc -> bMaxSlotIndex > 0)
-					break;
-
 				/* CCID and not ICCD */
 				if ((PROTOCOL_CCID == ccid_desc -> bInterfaceProtocol)
 					/* 3 end points */
@@ -1405,7 +1405,10 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 	/* Implement the PC/SC v2.02.07 Part 10 IOCTL mechanism */
 
 	/* Query for features */
-	if (CM_IOCTL_GET_FEATURE_REQUEST == dwControlCode)
+	/* 0x313520 is the Windows value for SCARD_CTL_CODE(3400)
+	 * This hack is needed for RDP applications */
+	if ((CM_IOCTL_GET_FEATURE_REQUEST == dwControlCode)
+		|| (0x313520 == dwControlCode))
 	{
 		unsigned int iBytesReturned = 0;
 		PCSC_TLV_STRUCTURE *pcsc_tlv = (PCSC_TLV_STRUCTURE *)RxBuffer;
